
from abc import ABC, abstractmethod


class IHandler(ABC):
    @abstractmethod
    def handle(self, message):
        """
        you must implement this method to handle the message
        """
        pass




class FuXLoggerException(Exception):
    """
    Base class for all FuXLogger exceptions.
    """
    pass

class RendererException(FuXLoggerException):
    """
    Exception raised when there is an error in the rendering process.
    """
    pass

class NotImplementedException(FuXLoggerException):
    """
    Exception raised when a method or function is not implemented yet.
    """
    pass

class InvalidConfigurationException(FuXLoggerException):
    """
    Exception raised when there is an error in the configuration file.
    """
    pass

class InvalidHandlerException(FuXLoggerException):
    """
    Exception raised when there is an error in the handler configuration.
    """
    pass

class InvalidEnvironmentException(FuXLoggerException):
    """
    Exception raised when there is an error in the environment configuration.
    example: if you open the 'is_async' option ,but you running in sync mode, you will get this exception.
    """
    pass


import enum


class LogLevel(enum.Enum):
    ON = float('-inf')
    TRACE = 0
    DEBUG = 1
    INFO = 2
    WARN = 3
    WARNING = WARN
    ERROR = 4
    FATAL = 5
    CRITICAL = FATAL
    OFF = float('inf')

    __level_to_str = {
        OFF: "OFF",
        TRACE: "TRACE",
        DEBUG: "DEBUG",
        INFO: "INFO",
        WARN: "WARN",
        ERROR: "ERROR",
        FATAL: "FATAL",
        ON: "ON"
    }

    __str_to_level = {
        "OFF": OFF,
        "TRACE": TRACE,
        "DEBUG": DEBUG,
        "INFO": INFO,
        "WARN": WARN,
        "WARNING": WARN,
        "ERROR": ERROR,
        "FATAL": FATAL,
        "CRITICAL": FATAL,
        "ON": ON
    }

    def __str__(self):
        return self.name
    
    @staticmethod
    def addlevel(level: dict[str, int]) -> None:
        for k, v in level.items():
            if k in LogLevel.__str_to_level:
                raise ValueError(f"Level {k} already exists")
            LogLevel.__str_to_level[k] = v
            LogLevel.__level_to_str[v] = k

    @staticmethod
    def levelToStr(level: "LogLevel") -> str:
        if level.value in LogLevel.__level_to_str:
            return LogLevel.__level_to_str[level] # type: ignore
        else:
            return "UNKNOWN"
    
    @staticmethod
    def strToLevel(level: str) -> "LogLevel":
        if level in LogLevel.__str_to_level:
            return LogLevel.__str_to_level[level]
        else:
            return LogLevel.OFF


import enum
import re
from .exceptions import RendererException

class Color(enum.Enum):
    """
    定义终端文本颜色的枚举类。
    """
    RED = '\033[31m'
    GREEN = '\033[32m'
    YELLOW = '\033[33m'
    BLUE = '\033[34m'
    MAGENTA = '\033[35m'
    CYAN = '\033[36m'
    WHITE = '\033[37m'
    PURPLE = '\033[95m'
    RESET = '\033[0m'
    GREY = '\033[90m'

class Font(enum.Enum):
    """
    定义终端文本字体样式的枚举类。
    """
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'
    BLINK = '\033[5m'
    REVERSE = '\033[7m'
    HIDE = '\033[8m'

def getColorDICT() -> dict[str, str]:
    """
    返回一个包含所有颜色枚举的字典。
    """
    colorDict = {color.name: color.value for color in Color}
    return colorDict

def getFontDICT() -> dict[str, str]:
    """
    返回一个包含所有字体样式枚举的字典。
    """
    fontDict = {font.name: font.value for font in Font}
    return fontDict

class Render:
    """
    提供文本渲染功能的类。
    """

    @staticmethod
    def render(text: str, color: Color, font: Font) -> str:
        """
        根据指定的颜色和字体样式渲染文本。

        :param text: 要渲染的文本。
        :param color: 颜色枚举值。
        :param font: 字体样式枚举值。
        :return: 渲染后的文本。
        """
        return f"{color.value}{font.value}{text}{Color.RESET.value}"
    
    @staticmethod
    def removeTags(text: str) -> str:
        """
        移除XML标签。
        
        :param text: 要移除标签的文本。
        :return: 移除标签后的文本。
        """
        pattern = r'<(\w+):(\w+)>(.*?)</(\w+):(\w+)>'
        return re.sub(pattern, r'\3', text, flags=re.DOTALL)
    
    @staticmethod
    def renderWithXML(text: str) -> str:
        """
        根据XML标签渲染文本。

        :param text: 要渲染的文本。
        :return: 渲染后的文本。
        :raises RendererException: 标签匹配错误。
        """
        colorDict = getColorDICT()
        fontDict = getFontDICT()
        pattern = r'<(\w+):(\w+)>(.*?)</(\w+):(\w+)>'
        def replaceFunc(match):
            start_color_name: str = match.group(1)
            start_font_name: str = match.group(2)
            content: str = match.group(3)
            end_color_name: str = match.group(4)
            end_font_name: str = match.group(5)

            # 检查开始和结束标签是否匹配
            if start_color_name.upper() != end_color_name.upper() or start_font_name.upper() != end_font_name.upper():
                raise RendererException(f"Mismatched tags: <{start_color_name}:{start_font_name}> does not match </{end_color_name}:{end_font_name}>")
            
            start_color = colorDict.get(start_color_name.upper())
            start_font = fontDict.get(start_font_name.upper())
            if start_color and start_font:
                return f"{start_color}{start_font}{content}{Color.RESET.value}"
            elif start_color:
                return f"{start_color}{content}{Color.RESET.value}"
            elif start_font:
                return f"{start_font}{content}{Color.RESET.value}"
            else:
                return content
        return re.sub(pattern, replaceFunc, text, flags=re.DOTALL)
    
if __name__ == '__main__':
    # text = "This is <purple:bold>purple</blue:bold> and <red:underline>red</red:underline> text."
    # print(Render.renderWithXML(text))
    # if you do this, you will get a exception: Mismatched tags: <purple:bold> does not match </blue:bold>
    pass
    



__version__ = "0.2.5"
__author__ = "<符玄(KOKOMI12345)>"
__email__ = "3072252442@qq.com"
__license__ = "MIT"
__copyright__ = "Copyright 2024 KOKOMI12345"
__python_version__ = "3.12.6"
__project_name__ = "FuXLogger"
__github_url__ = "https://github.com/KOKOMI12345/FuXLogger"



import warnings
from typing import Callable, Any, Optional, Type , Union
import inspect

class Deprecated:
    """
    ### Decorator for deprecated functions and classes.
    - 对弃用函数和类标记的装饰器
    
    ### three methods to how to use this decorators
    ### 此装饰器使用的三种方法
    
    - 1. 指定原因(input reason for deprecated functions)
    
    ```
    @Deprecated("this is a test")
    def test1():
        print("test1")
    ```
    
    - 2. 指定移除版本号(input deprecated version)
    
    ```
    @Deprecated("test", d_version="0.1.0", remove=True)
    def test2():
        print("test2")
    ```
    
    - 3. 指定新函数名(input new function name)
    
    ```
    @Deprecated("test", newFuncName="test4")
    def test3():
        print("test3")
    ```
    """

    def __init__(self, 
                 reason: str = "no reasons",
                 d_version: Optional[str] = None,
                 newName: Optional[str] = None,
                 remove: bool = False
                ) -> None:
        self.reason = reason
        self.d_version = d_version
        self.newName = newName
        self.remove = remove

    def __call__(self, obj: Union[Callable[..., Any], Type]) -> Union[Callable[..., Any], Type]:
        """
        Handle function or class decoration.
        """
        if inspect.isclass(obj):
            return self._decorate_class(obj)
        elif inspect.isfunction(obj):
            return self._decorate_function(obj)
        else:
            raise TypeError("Deprecated can only be used on functions or classes.")

    def _decorate_function(self, func: Callable[..., Any]) -> Callable[..., Any]:
        """
        Decorates a function.
        """
        func_signature = inspect.signature(func)

        def wrapper(*args: Any, **kwargs: Any) -> Any:
            if not func.__name__ == "new_init": 
                self._warn("function", func.__name__)
            return func(*args, **kwargs)

        wrapper.__signature__ = func_signature
        return wrapper

    def _decorate_class(self, cls: Type) -> Type:
        """
        Decorates a class.
        """
        original_init = cls.__init__

        def new_init(instance, *args: Any, **kwargs: Any) -> None:
            self._warn("class", cls.__name__)
            original_init(instance, *args, **kwargs)

        cls.__init__ = new_init

        for attr_name, attr_value in cls.__dict__.items():
            if callable(attr_value):
                setattr(cls, attr_name, self._decorate_function(attr_value))
        
        return cls

    def _warn(self, is_what: str, name: str) -> None:
        """
        Issues the warning based on the configuration.
        """
        formatted_reason = f"Deprecated as {self.reason}, Deprecated {is_what}: {name}"
        if self.d_version:
            formatted_reason += f", will be removed in version {self.d_version}"

        if self.remove:
            stacklevel = 3 if is_what == "function" else 4
            warnings.warn(f"{formatted_reason}.", DeprecationWarning, stacklevel=stacklevel)
        elif self.newName:
            stacklevel = 3 if is_what == "function" else 4
            warnings.warn(f"{formatted_reason}, use {self.newName} instead.", FutureWarning, stacklevel=stacklevel)
        else:
            stacklevel = 3 if is_what == "function" else 4
            warnings.warn(formatted_reason, DeprecationWarning, stacklevel=stacklevel)


if __name__ == "__main__":
    
    @Deprecated("this is a test")
    class Test: ...

    @Deprecated("this is a test")
    def test():
        print("test")

    test()
    testClass = Test()


import sys

# 通过判断版本来判断类型注解的支持性
if sys.version_info >= (3, 12):
    from typing import Union
    type Message = Union[str, bytes]
else:
    from typing import TypeVar
    Message = TypeVar('Message', str, bytes)


from .LogBody import LogRecord
import datetime

class LogFormatter:
    """
    日志格式化器
    ## 新日志库的格式如下:
    - {time}: 时间
    - {level}: 日志级别的INT值
    - {levelName}: 日志级别的名称
    - {message}: 日志信息
    - {module}: 日志所在模块
    - {file}: 日志所在文件名
    - {line}: 日志所在行号
    - {function}: 日志所在函数名
    - {processid}: 进程ID
    - {threadid}: 线程ID
    - {threadName}: 线程名
    - {processName}: 进程名
    - {pathname}: 日志所在文件路径
    - {workdir}: 当前工作目录
    - {timestamp}: 日志本地时间戳
    - {utctime}: 日志UTC时间戳
    - {Stack}: 日志堆栈信息
    """
    def __init__(self, format: str, datefmt: str = "%Y-%m-%d %H:%M:%S") -> None:
        self._format = format
        self._datefmt = datefmt

    def setFormatter(self, format: str, datefmt: str = "%Y-%m-%d %H:%M:%S") -> None:
        """
        设置日志格式化器
        :param format: 日志格式
        :param datefmt: 时间格式
        :return: None
        """
        self._format = format
        self._datefmt = datefmt

    def format(self, record: LogRecord) -> str:
        """
        格式化日志记录
        :param record: 日志记录字典
        :return: 格式化后的日志字符串
        """
        record_copy = record.ToDict()
        record_copy['timestamp'] = datetime.datetime.fromtimestamp(record.timestamp)
        record_copy['time'] = record_copy['timestamp'].strftime(self._datefmt)
        formatted_msg = self._format.format(**record_copy)
        return formatted_msg



from . import StreamHandler , LogFormatter , LogManager , LogLevel

my_log_fmt = LogFormatter("{time} | {levelName:<7} | {module}:{function} | {file}:{line:02} | {message}")

logger = LogManager.getLogger("main", my_log_fmt,enqueue=True)

console_handler = StreamHandler("console", LogLevel.ON, my_log_fmt,colorize=True, enableXMLRender=True)

logger.addHandler(console_handler)

__all__ = ["logger"]



from .excformat import ExtractException , GetStackTrace
from .color import Color , Font, Render
from .helpers import Deprecated

__all__ = [
    'ExtractException', 'Deprecated', 
    'GetStackTrace', 
    'Color', 'Font', 
    'Render'
]



from .loglevel import LogLevel
from ..utils.color import Color

_logColors = {
    LogLevel.TRACE : Color.GREY,
    LogLevel.DEBUG : Color.BLUE,
    LogLevel.INFO : Color.CYAN,
    LogLevel.WARN : Color.YELLOW,
    LogLevel.ERROR : Color.RED,
    LogLevel.FATAL : Color.MAGENTA
}



from .loglevel import LogLevel
from dataclasses import dataclass
from ..utils.types import Message

@dataclass
class LogRecord:
    """
    这个类是用来存储日志信息的类
    """
    name: str
    level: LogLevel
    levelName: str
    message: Message
    time: str
    timestamp: float
    utctime: float
    module: str
    function: str
    line: int
    file: str
    pathname: str
    workdir: str
    processid: int
    threadid: int
    threadName: str
    processName: str
    stack_info: str

    def __str__(self) -> str:
        return f"""
        Name: {self.name}, 
        Level: {self.level}, 
        LevelName: {self.levelName}, 
        Message: {self.message}, 
        Time: {self.time}, 
        Timestamp: {self.timestamp}, 
        UTCTime: {self.utctime}, 
        Module: {self.module}, 
        Function: {self.function}, 
        Line: {self.line}, 
        File: {self.file}, 
        PathName: {self.pathname}, 
        workdir: {self.workdir}, 
        ProcessID: {self.processid}, 
        ThreadID: {self.threadid}, 
        ThreadName: {self.threadName}, 
        ProcessName: {self.processName}, 
        StackInfo: {self.stack_info}
        """
    
    def getMessage(self) -> Message:
        return self.message
    
    def ToDict(self) -> dict:
        return {
            "name": self.name,
            "level": self.level,
            "levelName": self.levelName,
            "message": self.message,
            "time": self.timestamp,
            "timestamp": self.timestamp,
            "utctime": self.utctime,
            "module": self.module,
            "function": self.function,
            "line": self.line,
            "file": self.file,
            "pathname": self.pathname,
            "workdir": self.workdir,
            "processid": self.processid,
            "threadid": self.threadid,
            "threadName": self.threadName,
            "processName": self.processName,
            "Stack": self.stack_info
        }
    
    def ToJson(self) -> str:
        import json
        return json.dumps(self.ToDict())
    
    def ToTuple(self) -> tuple:
        return (
            self.name,
            self.level,
            self.levelName,
            self.message,
            self.time,
            self.timestamp,
            self.utctime,
            self.module,
            self.function,
            self.line,
            self.file,
            self.pathname,
            self.workdir,
            self.processid,
            self.threadid,
            self.threadName,
            self.processName,
            self.stack_info
        )


from .logger import Logger
from .formatter import LogFormatter
import threading

class LogManager:

    loggers: dict[str, Logger] = {}
    dftformatter: LogFormatter = LogFormatter("{time} | {level:<7} | {message}")
    lock = threading.Lock()

    @staticmethod
    def getLogger(name: str, formatter: LogFormatter = dftformatter, enqueue: bool = False, is_async: bool = False) -> Logger:
        """
        获取一个日志记录器对象,注意,如果is_async为True,则确保必须再异步环境内调用此方法,否则会报错
        """
        with LogManager.lock:
            if LogManager.loggers.get(name) is None:
                LogManager.loggers[name] = Logger(name=name, enqueue=enqueue, is_async=is_async)
            return LogManager.loggers[name]


from ..utils.interfaces import IHandler
from .LogBody import LogRecord
from .formatter import LogFormatter
from .loglevel import LogLevel
from .logcolor import _logColors as LogColor
from ..utils.color import Font
from ..utils import Render
from ..utils.exceptions import NotImplementedException
import threading
import sys
import socket
import queue

class Handler(IHandler):
    def __init__(self, 
        name: str,
        level: LogLevel, 
        formatter: LogFormatter
    ) -> None:
        self.name = name
        self.level = level
        self.formatter = formatter

    def handle(self, record: LogRecord) -> None:
        raise NotImplementedException("handle method not implemented")

class StreamHandler(Handler):
    def __init__(self, name: str, 
        level: LogLevel, 
        formatter: LogFormatter, 
        stream=sys.stdout, 
        colorize: bool = False,
        enableXMLRender: bool = False
    ) -> None:
        super().__init__(name, level, formatter)
        self.stream = stream
        self.colorize = colorize
        self.colors = LogColor
        self.enableXMLRender = enableXMLRender

    def write(self, message: str) -> None:
        self.stream.write(message)
        self.stream.flush()

    def handle(self, record: LogRecord) -> None:
        if record.level.value >= self.level.value:
            if self.colorize and self.enableXMLRender:
                color = self.colors.get(record.level)
                record.message = Render.renderWithXML(record.message)
                renderedMsg = Render.render(self.formatter.format(record), color, Font.BOLD) # type: ignore
                self.write(f"{renderedMsg}\n")
            elif self.colorize:
                color = self.colors.get(record.level)
                renderedMsg = Render.render(self.formatter.format(record), color, Font.BOLD) # type: ignore
                self.write(f"{renderedMsg}\n")
            else:
                self.write(f"{self.formatter.format(record)}\n")

class SocketHandler(Handler):
    """
    用户可以指定一个socket地址，然后将日志发送到指定的socket地址
    """
    def __init__(self, name: str, 
        level: LogLevel, 
        formatter: LogFormatter, 
        host: str, 
        port: int, 
        timeout: int = 1
    ) -> None:
        super().__init__(name, level, formatter)
        self.host = host
        self.port = port
        self.timeout = timeout
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.settimeout(self.timeout)
        self.process_queue = queue.Queue()
        self.failed_cache = queue.Queue()
        self.thread = threading.Thread(target=self.__handle)
        self.thread.start()

    def close(self) -> None:
        if self.sock:
           self.sock.close()

    def __send_message(self, message: str) -> None:
        try:
            self.sock.connect((self.host, self.port))
            self.sock.sendall(message.encode())
        except Exception as e:
            print(f"Error while sending message to socket: {e}")
            self.failed_cache.put(message)

    def __handle_failed_cache(self) -> None:
        while not self.failed_cache.empty():
            message = self.failed_cache.get()
            self.__send_message(message)

    def __handle(self) -> None:
        while threading.main_thread().is_alive():
            try:
                message = self.process_queue.get(block=False, timeout=10) # 指定超时时间避免忙等待
                self.__send_message(message)
            except queue.Empty:
                self.__handle_failed_cache()
            except Exception as e:
                print(f"Error while handling message: {e}")
                self.failed_cache.put(message)
        self.close() # close socket when main thread is not alive

    def handle(self, record: LogRecord) -> None:
        if record.level.value >= self.level.value:
            message = Render.removeTags(self.formatter.format(record))
            self.process_queue.put(message)
            
    

class FileHandler(Handler):
    def __init__(self, name: str, 
        level: LogLevel, 
        formatter: LogFormatter, 
        filename: str, 
        mode: str = "a", 
        encoding: str = "utf-8",
    ) -> None:
        super().__init__(name, level, formatter)
        self.filename = filename
        self.mode = mode
        self.encoding = encoding
        self.lock = threading.Lock()

    def handle(self, record: LogRecord) -> None:
        if record.level.value >= self.level.value:
            with self.lock:
                with open(self.filename, self.mode, encoding=self.encoding) as f:
                    f.write(f"{Render.removeTags(self.formatter.format(record))}\n")



from .core import LogManager , LogFormatter, StreamHandler , LogLevel , Handler
from .core.handlers import FileHandler , SocketHandler

__all__ = [
    'LogManager', 
    'LogFormatter', 'StreamHandler', "FileHandler", "SocketHandler",
    'LogLevel', 'Handler'
]



from datetime import datetime , timezone

def getUTCDateTime() -> float:
    """
    Returns the current UTC datetime in seconds since epoch.
    """
    return datetime.now(timezone.utc).timestamp()

def getLocalDateTime() -> float:
    """
    Returns the current local datetime in seconds since epoch.
    """
    return datetime.now().timestamp()


import traceback
import multiprocessing
import threading
import sys
from typing import Optional

def format_stack_trace(exctype, value, tb, nested: bool = False,stackTraceMode: bool = False) -> str:
    tb_list = traceback.extract_tb(tb)

    if nested:
        exception_info = f"{exctype.__name__}: {value}\n"
    else:
        if not stackTraceMode:
            exception_info = f"Exception in process: {multiprocessing.current_process().name}, thread: {threading.current_thread().name}; {exctype.__name__}: {value}\n"
            exception_info += "Traceback (most recent call last):\n"
        else:
            exception_info = "This is just a stack trace\n"
            exception_info += "Traceback (most recent call last):\n"

    for filename, lineno, funcname, line in tb_list:
        exception_info += f"  at {funcname} in ({filename}:{lineno})\n"
    
    # 检查是否有原因和其他信息
    cause = getattr(value, '__cause__', None)
    context = getattr(value, '__context__', None)
    
    if cause:
        exception_info += "Caused by: "
        exception_info += format_stack_trace(type(cause), cause, cause.__traceback__, nested=True)
    if context and not cause:
        exception_info += "original exception: "
        exception_info += format_stack_trace(type(context), context, context.__traceback__, nested=True)
    
    return exception_info

def ExtractException(exctype, e, tb) -> Optional[str]:
    # 获取回溯信息并格式化为字符串
    tb_str = format_stack_trace(exctype, e, tb)
    
    # 记录异常信息到日志
    exception_info = ""
    exception_info += tb_str
    # 返回异常信息
    return exception_info

def GetStackTrace(vokedepth: int = 1) -> str:
    """
    获取堆栈跟踪信息
    """
    # 获取当前调用栈信息的前两层
    stack = traceback.extract_stack(limit=vokedepth)
    stack_info = "Stack Trace:\n"
    for frame in stack[:-vokedepth+1]:
        filename = frame.filename
        line = frame.lineno
        funcname = frame.name
        stack_info += f"  at {funcname} in ({filename}:{line})\n"
    return stack_info


from setuptools import setup, find_packages
from FuXLogger.__metadata__ import __version__ , __author__ , __project_name__ , __email__ , __license__ , __github_url__

def load_requirements(filepath: str = "requirements.txt") -> list[str]:
    try:
       with open(filepath, 'r') as f:
           return [line.strip() for line in f.readlines()]
    except FileNotFoundError:
        return []
    except Exception:
        return []

setup(
    name=__project_name__,
    version=__version__,
    packages=find_packages(),
    author=__author__,
    author_email=__email__,
    description='A useful logging library for Python',
    long_description=open('Readme.md','r',encoding='utf-8').read(),
    long_description_content_type='text/markdown',
    url=__github_url__,
    requires=load_requirements(),  # 依赖
    license=__license__,
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)



import asyncio
from .loglevel import LogLevel
from .handlers import Handler
from .LogBody import LogRecord
from ..utils import ExtractException
from ..utils.excformat import GetStackTrace
from ..utils.timeutil import getLocalDateTime, getUTCDateTime
from ..utils.exceptions import InvalidConfigurationException
from ..utils.types import Message
from ..utils.exceptions import InvalidEnvironmentException
import threading
import multiprocessing
import queue
import os
import sys
import inspect

class Logger:
    def __init__(self, name: str, level: LogLevel = LogLevel.ON, handlers: set[Handler] = set(), enqueue: bool = False, is_async: bool = False):
        self.name: str = name
        self.level: LogLevel = level
        self.handlers: set[Handler] = handlers
        self.enqueue: bool = enqueue
        self.is_async: bool = is_async
        if enqueue and is_async:
            raise InvalidConfigurationException("Cannot use enqueue and is_async at the same time")
        if enqueue:
            self.queue = queue.Queue()
            self.log_thread = threading.Thread(target=self.__enqueueHandler)
            self.log_thread.start()
        elif is_async:
            try:
                self.async_queue = asyncio.Queue()
                self.loop = asyncio.get_running_loop()
                self.start_async_logging()
            except RuntimeError as caused_by:
                raise InvalidEnvironmentException("Cannot use is_async outside of an asyncio event loop") from caused_by

    def start_async_logging(self):
        if not self.is_async:
            return
        self.log_task = self.loop.create_task(self.__async_enqueueHandler())

    def stop_async_logging(self):
        if self.is_async and self.log_task:
            self.log_task.cancel()
            try:
                self.loop.run_until_complete(self.log_task)
            except asyncio.CancelledError:
                pass
            except RuntimeError:
                pass
            self.log_task = None

    def __del__(self):
        if self.is_async and not self.enqueue:
            try:
              self.stop_async_logging()
            except AttributeError:
                pass

    def addLevel(self, level: dict[str, int]) -> None:
        LogLevel.addlevel(level)

    def setLevel(self, level: LogLevel) -> None:
        self.level = level

    def addHandler(self, handler: Handler) -> None:
        self.handlers.add(handler)

    def removeHandler(self, handler: Handler) -> None:
        self.handlers.remove(handler)

    async def __async_enqueueHandler(self) -> None:
        while True:
            try:
                async with asyncio.Lock():
                    record = await asyncio.wait_for(self.async_queue.get(), timeout=0.1)
                    for handler in self.handlers:
                        await asyncio.to_thread(handler.handle, record)
                    self.async_queue.task_done()
            except asyncio.TimeoutError:
                continue
            except asyncio.CancelledError:
                break

    def __enqueueHandler(self) -> None:
        while True:
            try:
                record = self.queue.get(timeout=0.1)
                for handler in self.handlers:
                    handler.handle(record)
            except queue.Empty:
                if not threading.main_thread().is_alive():
                    break

    def __makeRecord(self, message: Message, level: LogLevel) -> LogRecord:
        frame = inspect.currentframe().f_back.f_back.f_back  # type: ignore
        levelname = level.name
        current_module = inspect.getmodule(frame)
        module = current_module.__name__ if current_module else "__main__"
        return LogRecord(
            name=self.name,
            level=level,
            levelName=levelname,
            time="", # 留给后面格式化处理
            timestamp=getLocalDateTime(),
            utctime=getUTCDateTime(),
            threadid=threading.get_ident(),
            processid=multiprocessing.current_process().pid,  # type: ignore
            processName=multiprocessing.current_process().name,
            threadName=threading.current_thread().name,
            stack_info=GetStackTrace(5),
            file=os.path.basename(frame.f_code.co_filename),  # type: ignore
            pathname=frame.f_code.co_filename, # type: ignore
            workdir=os.getcwd(),
            line=frame.f_lineno,  # type: ignore
            function=frame.f_code.co_name,  # type: ignore
            module=module,  # type: ignore
            message=message
        )

    def __log(self, level: LogLevel, message: Message) -> None:
        """
        记录一个日志的内部实现
        """
        if self.enqueue:
            self.queue.put(self.__makeRecord(message, level))
        elif self.is_async:
            self.async_queue.put_nowait(self.__makeRecord(message, level))
        else:
            for handler in self.handlers:
                handler.handle(self.__makeRecord(message, level))

    def exception(self, message: str, is_fatal: bool = False) -> None:
        """
        记录异常信息到日志,如果没有异常发生,这个只会记录一个ERROR级别的日志(取决于是否设置了is_fatal),如果有,则会带上堆栈信息
        - is_fatal: 是否是致命错误,如果是,则会记录FATAL级别的日志,否则记录ERROR级别的日志
        """
        exc_type, exc_value, exc_traceback = sys.exc_info()
        if exc_traceback is not None:
            exc_info = ExtractException(exc_type, exc_value, exc_traceback)
            if is_fatal:
                self.__log(LogLevel.FATAL, f"{message}\n{exc_info}")
            else:
                self.__log(LogLevel.ERROR, f"{message}\n{exc_info}")
        elif is_fatal:
            self.__log(LogLevel.FATAL, message)
        else:
            self.__log(LogLevel.ERROR, message)

    def log(self, level: LogLevel, message: Message) -> None:
        """
        记录一个 {level} 级别的日志
        """
        self.__log(level, message)

    def trace(self, message: Message) -> None:
        """
        记录一个 TRACE 级别的日志
        """
        self.__log(LogLevel.TRACE, message)

    def debug(self, message: Message) -> None:
        """
        记录一个 DEBUG 级别的日志
        """
        self.__log(LogLevel.DEBUG, message)

    def info(self, message: Message) -> None:
        """
        记录一个 INFO 级别的日志
        """
        self.__log(LogLevel.INFO, message)

    def warning(self, message: Message) -> None:
        """
        记录一个 WARN 级别的日志
        """
        self.__log(LogLevel.WARN, message)

    def error(self, message: Message) -> None:
        """
        记录一个 ERROR 级别的日志
        """
        self.__log(LogLevel.ERROR, message)

    def fatal(self, message: Message) -> None:
        """
        记录一个 FATAL 级别的日志
        """
        self.__log(LogLevel.FATAL, message)



from .handlers import StreamHandler , Handler
from .logger import Logger
from .loglevel import LogLevel
from .formatter import LogFormatter
from .LogManager import LogManager

__all__ = ['StreamHandler', 'Handler', 'Logger', 'LogFormatter', 'LogManager', 'LogLevel']